@@ -1,6 +1,6 @@
 /********************************************
 mawk.h
-copyright 2008-2016,2019 Thomas E. Dickey
+copyright 2008-2020,2021 Thomas E. Dickey
 copyright 1991-1995,1996 Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: mawk.h,v 1.54 2019/02/02 01:00:14 tom Exp $
+ * $MawkId: mawk.h,v 1.62 2021/05/28 22:02:11 tom Exp $
  */
 
 /*  mawk.h  */
@@ -32,6 +32,12 @@ the GNU General Public License, version 2, 1991.
 
 #include "types.h"
 
+#ifdef HAVE_STDNORETURN_H
+#include <stdnoreturn.h>
+#undef GCC_NORETURN
+#define GCC_NORETURN STDC_NORETURN
+#endif
+
 #ifndef GCC_NORETURN
 #define GCC_NORETURN		/* nothing */
 #endif
@@ -63,7 +69,13 @@ extern int dump_RE;
 #define USE_BINMODE 0
 #endif
 
-extern short posix_space_flag, interactive_flag;
+extern short interactive_flag;
+extern short posix_space_flag;
+extern short traditional_flag;
+
+#ifndef NO_INTERVAL_EXPR
+extern short repetitions_flag;
+#endif
 
 /*----------------
  *  GLOBAL VARIABLES
@@ -99,6 +111,8 @@ extern char decimal_dot;
 extern const char *progname;	/* for error messages */
 extern unsigned rt_nr, rt_fnr;	/* ditto */
 
+#define TABLESIZE(name) (sizeof(name)/sizeof(name[0]))
+
 /* macro to test the type of two adjacent cells */
 #define TEST2(cp)  (mpow2[(cp)->type]+mpow2[((cp)+1)->type])
 
@@ -129,9 +143,11 @@ extern void cast_for_split(CELL *);
 extern void check_strnum(CELL *);
 extern void cast_to_REPL(CELL *);
 extern Int d_to_I(double);
-extern UInt d_to_U(double d);
+extern Long d_to_L(double);
+extern ULong d_to_UL(double d);
 
 #define d_to_i(d)     ((int)d_to_I(d))
+#define d_to_l(d)     ((long)d_to_L(d))
 
 extern int test(CELL *);	/* test for null non-null */
 extern CELL *cellcpy(CELL *, CELL *);
@@ -139,8 +155,8 @@ extern CELL *repl_cpy(CELL *, CELL *);
 extern void DB_cell_destroy(CELL *);
 extern void overflow(const char *, unsigned);
 extern void rt_overflow(const char *, unsigned);
-extern void rt_error(const char *,...) GCC_NORETURN GCC_PRINTFLIKE(1,2);
-extern void mawk_exit(int) GCC_NORETURN;
+extern GCC_NORETURN void rt_error(const char *,...) GCC_PRINTFLIKE(1,2);
+extern GCC_NORETURN void mawk_exit(int);
 extern void da(INST *, FILE *);
 extern INST *da_this(INST *, INST *, FILE *);
 extern char *rm_escape(char *, size_t *);
@@ -152,12 +168,14 @@ extern char *str_str(char *, size_t, const char *, size_t);
 #endif
 
 extern void parse(void);
-extern int yylex(void);
-extern int yyparse(void);
-extern void yyerror(const char *);
 extern void scan_cleanup(void);
 
-extern void bozo(const char *) GCC_NORETURN;
+#ifndef YYBYACC 
+extern int yylex(void);
+#endif
+extern void yyerror(const char *);
+
+extern GCC_NORETURN void bozo(const char *);
 extern void errmsg(int, const char *,...) GCC_PRINTFLIKE(2,3);
 extern void compile_error(const char *,...) GCC_PRINTFLIKE(1,2);
 
